//! Handle requests for cover images.

use std::sync::Arc;

use poem::{
    error::NotFoundError,
    handler,
    web::{headers::ContentType, Data, Path},
    Body, IntoResponse, Response,
};
use tokio::fs::File;
use tokio_util::io::ReaderStream;

use crate::{app_state::AppState, handlers::error::HandlerError};

/// Handle a request for the cover image of book with id `id`.
#[handler]
pub async fn handler(id: Path<u64>, state: Data<&Arc<AppState>>) -> Result<Response, poem::Error> {
    let book = state
        .calibre
        .scalar_book(*id)
        .map_err(HandlerError::DataError)?;
    let cover_path = state.config.library_path.join(book.path).join("cover.jpg");
    let mut cover = File::open(cover_path).await.map_err(|_| NotFoundError)?;
    let stream = ReaderStream::new(cover);
    let body = Body::from_bytes_stream(stream);

    Ok(body
        .with_content_type(ContentType::jpeg().to_string())
        .with_header("Content-Disposition", "filename=\"cover.jpg\"")
        .into_response())
}
